"""Abstract base class for executors."""

from abc import ABC, abstractmethod

from concierge.task import Task


class BaseExecutor(ABC):
    """Abstract base class for executors."""

    def __init__(self, *args, **kwargs) -> None:
        """Initialize the executor with the given arguments."""

    @abstractmethod
    def run_task(self, task: Task) -> None:
        """Run the given task."""

    @abstractmethod
    def stop_task(self, uuid: str) -> None:
        """Stop the task with the given UUID."""

    @abstractmethod
    def list_task_uuids(self) -> list[str]:
        """Return a list of UUIDs of all running tasks."""
